from ._bar_count import _get as bar_count
from ._box import _get as box
from ._histogram_line import _get as histogram_line
from ._imshow import _get as imshow
from ._probability import _get as probability
from ._table import _get as table

